/**
* Very basic robot movment control.  Needs motor speed control implementing on target hardware
*
* Uses ControlP5 library from http://www.sojamo.de/libraries/controlP5/
*
*/

import controlP5.*;
import processing.serial.*;

// Serial port for communications with robot
Serial myPort;

// Robot Image
PImage img;

// 2D Slider for directionaly control
ControlP5 cp5;
Slider2D s;

// Motor states
int motorRight = 0;
int motorLeft = 0;
int previousMotorRight = 0;
int previousMotorLeft = 0;

void setup() {
  int i;
  
  size(700,400);
  
  img = loadImage("wk15-test-robot-small.png");
  
  cp5 = new ControlP5(this);
  s = cp5.addSlider2D("Robot Control")
         .setPosition(30,40)
         .setSize(300,300)
         .setArrayValue(new float[] {50, 50})
         .setDecimalPrecision(0)
         //.setImage(loadImage("wk15-test-robot.jpg"), Controller.DEFAULT)
         //.setImages(loadImage("wk15-test-robot.jpg"), loadImage("wk15-test-robot.jpg"), loadImage("wk15-test-robot.jpg"))
         //.disableCrosshair()
         .updateSize()
         ;
         
    cp5.addButton("Stop")
     .setValue(0)
     .setPosition(30,10)
     .setSize(100,19)
     ;

    cp5.addTextlabel("description")
                    .setText("Drag the cross hair to make the robot move.\n\nUse the 'Stop' button to halt all movement.")
                    .setPosition(350,40)
                    .setWidth(300)
                    .setHeight(300)
                    .setColorValue(0xffffff00)
                    .setMultiline(true)
                    .setFont(createFont("arial",12))
                    ;
  smooth();
  
  // On Mac open 1stport we find with "usbserial" in the name
  String portName = null; //Serial.list()[0];
  for(i=0; i<Serial.list().length; i++)
  {
    String[] match_test = match(Serial.list()[i], "usbserial");
    println("Serial " + i + ": " + Serial.list()[i]);
    if((match_test != null) && (portName == null))
    {
      // Serial port found with "usbserial" in the name
      portName = Serial.list()[i];
    }
  }
  
  if(portName == null)
    portName = Serial.list()[0];
  
  println("Opening: " + portName);
  myPort = new Serial(this, portName, 9600);
  myPort.buffer(1);
}

float cnt;
void draw() {
  /*
  background(0);
  pushMatrix();
  translate(160,140);
  noStroke();
  fill(50);
  rect(0, -100, 400,200);
  strokeWeight(1);
  line(0,0,200, 0);
  stroke(255);
  
  for(int i=1;i<400;i++) {
    float y0 = cos(map(i-1,0,s.arrayValue()[0],-PI,PI)) * s.arrayValue()[1]; 
    float y1 = cos(map(i,0,s.arrayValue()[0],-PI,PI)) * s.arrayValue()[1];
    line((i-1),y0,i,y1);
  }
  
  popMatrix();
  */
  
  image(img,350,100);
  
  // Slowly move crosshair back to center
  if( s.arrayValue()[0] > 50) s.arrayValue()[0] -= 1;
  if( s.arrayValue()[0] < 50) s.arrayValue()[0] += 1;
  s.setArrayValue(s.arrayValue());
  
  // Send data via serial port
  // When moving forward and turning just stop inside motor.  When moving backwards run motor backwards.
  // Need to implement speed control on robot
  //println(s.arrayValue()[1]);
  if(s.arrayValue()[1] < 40)
  {
    // Forward
    if(s.arrayValue()[0] < 40)
    {
      // Turn left
      //println("Turn Left");
      motorLeft = 0;
      motorRight = 1;
      //myPort.write('s'); // left stop
      //myPort.write(0xff); // Insert idle time on the serial line
      //myPort.write('o'); // right forward
    }
    else if(s.arrayValue()[0] > 60)
    {
      // Turn right
      //println("Turn Right");
      motorLeft = 1;
      motorRight = 0;
      //myPort.write('k'); // right stop
      //myPort.write(0xff); // Insert idle time on the serial line
      //myPort.write('w'); // left forward
    }
    else
    {
      // Forward
      //println("Full Steam Ahead");
      motorLeft = 1;
      motorRight = 1;
      //myPort.write('o'); // right forward
      //myPort.write(0xff); // Insert idle time on the serial line
      //myPort.write('w'); // left forward
    }
  }
  else if(s.arrayValue()[1] > 60)
  {
    // Reverse
    if(s.arrayValue()[0] < 40)
    {
      //println("Reverse Left");
      motorLeft = 1;
      motorRight = -1;
      //myPort.write('w');
      //myPort.write(0xff); // Insert idle time on the serial line
      //myPort.write('m');
    }
    else if(s.arrayValue()[0] > 60)
    {
      //println("Reverse Right");
      motorLeft = -1;
      motorRight = 1;
      //myPort.write('x');
      //myPort.write(0xff); // Insert idle time on the serial line
      //myPort.write('o');
    }
    else
    {
      //println("Reverse");
      motorLeft = -1;
      motorRight = -1;
      //myPort.write('x');
      //myPort.write(0xff); // Insert idle time on the serial line
      //myPort.write('m');
    }
  }
  else
  {
    // Stop
    //println("Stop!");
    motorLeft = 0;
    motorRight = 0;
    //myPort.write('k');
    //myPort.write(0xff); // Insert idle time on the serial line
    //myPort.write('s');
  }
  
  //println("motor " + motorLeft + ":" + motorRight + "   previous " + previousMotorLeft + ":" + previousMotorRight);
  
  // Send commands if motor states have changed
  if(motorLeft != previousMotorLeft)
  {
    char out = 's';
    
    previousMotorLeft = motorLeft;
    switch(motorLeft)
    {
      case 1:
      out = 'w';
      break;
      case 0:
      default:
      out = 's';
      break;
      case -1:
      out = 'x';
      break;
    }
    
    println("Left Motor: " + out);
    myPort.write(out);
    myPort.write(0xff);
  }
  
  // Insert idle time on the serial line
  //myPort.write(0xff);

  if(motorRight != previousMotorRight)
  {
    char out = 'k';
    
    previousMotorRight = motorRight;
    switch(motorRight)
    {
      case 1:
      out = 'o';
      break;
      case 0:
      default:
      out = 'k';
      break;
      case -1:
      out = 'm';
      break;
    }
    
    println("Right Motor: " + out);
    myPort.write(out);
  }

  // Insert idle time on the serial line
  //myPort.write(0xff);
}


public void Stop(int theValue) {
  // Stop button pressed, reset 2D slider to center
  s.arrayValue()[0] = 50;
  s.arrayValue()[1] = 50;
  s.setArrayValue(s.arrayValue());
}


